# jupyteranalytics.extension
# Implements the required notebook and server extension functions.
#
# Author:   PingThings
# Created:  Tue Nov 26 11:54:33 2019 -0500
#
# Copyright (C) 2019 PingThings, Inc.
# For license information, see LICENSE.txt
#
# ID: __init__.py [] benjamin@pingthins.io $

"""
Implements the required notebook and server extension functions.
"""

import os

from notebook.services.config import ConfigManager
from traitlets.config import Configurable
from traitlets import Unicode, default


CONFIG_SECTION = "common"
GOOGLE_ANALYTICS_TRACKING_ID = "GOOGLE_ANALYTICS_TRACKING_ID"


__all__ = [
    "_jupyter_server_extension_paths",
    "_jupyter_nbextension_paths",
    "load_jupyter_server_extension",
    "GoogleAnalytics",
    "CONFIG_SECTION",
]


def _jupyter_server_extension_paths():
    """Declare server extensions provided by this package."""
    return [{
        "module": "jupyteranalytics"
    }]


def _jupyter_nbextension_paths():
    """Declare Jupyter extension entry points."""
    return [{
        # Load this on all the pages!
        "section": CONFIG_SECTION,
        # Path relative to the jupyteranalytics directory
        "src": "static",
        # Directory in the nbextension/ namespace
        "dest": "jupyteranalytics",
        # Javascript also in the nbextension/ namespace
        "require": "jupyteranalytics/main"
    }]


def load_jupyter_server_extension(nbapp):
    """Configure and load the extension"""

    # Get the config from the command line or environment if available
    ga = GoogleAnalytics(parent=nbapp)
    ga.setup_config()

    # Check to see if the tracking ID is set
    config = nbapp.config_manager.get(CONFIG_SECTION)
    if config.get("GoogleAnalytics", {}).get("tracking_id", None):
        nbapp.log.info("Google Analytics enabled with Tracking ID")
    else:
        nbapp.log.info("Google Analytics is disabled, no Tracking ID specified")


class GoogleAnalytics(Configurable):
    """
    Configures the Google Analytics Tracking ID from the command line for a single
    session if specified with the --GoogleAnalytics.tracking_id flag.
    """

    tracking_id = Unicode(
        None,
        allow_none=True,
        help="The Google Analytics Tracking ID, usually UA-#########-#.",
        config=True
    )

    @default('tracking_id')
    def _default_tracking_id(self):
        return os.getenv(GOOGLE_ANALYTICS_TRACKING_ID, None)

    def setup_config(self):
        """
        Updates the config for the duration of the user session.
        """
        cm = ConfigManager()
        cm.update(
            CONFIG_SECTION,
            {
                'GoogleAnalytics': {
                    'tracking_id': self.tracking_id
                }
            }
        )



# jupyteranalytics.version
# Maintains version and package information for deployment.
#
# Author:   PingThings
# Created:  Tue Nov 26 11:52:24 2019 -0500
#
# For license information, see LICENSE.txt
# ID: version.py [] benjamin@pingthings.io $

"""
Maintains version and package information for deployment.
"""

##########################################################################
## Module Info
##########################################################################

__version_info__ = {
    'major': 1,
    'minor': 1,
    'micro': 0,
    'releaselevel': 'final',
    'serial': 2,
}

##########################################################################
## Helper Functions
##########################################################################

def get_version(short=False):
    """
    Prints the version.
    """
    assert __version_info__['releaselevel'] in ('alpha', 'beta', 'final')
    vers = ["%(major)i.%(minor)i" % __version_info__, ]
    if __version_info__['micro']:
        vers.append(".%(micro)i" % __version_info__)
    if __version_info__['releaselevel'] != 'final' and not short:
        vers.append('%s%i' % (__version_info__['releaselevel'][0],
                              __version_info__['serial']))
    return ''.join(vers)



#!/usr/bin/env python
# setup
# Installs and maintains the Python package
#
# Author:   PingThings
# Created:  Tue Nov 26 11:52:24 2019 -0500
#
# For license information, see LICENSE.txt
# ID: setup.py [] benjamin@pingthings.io $

"""
Installs and maintains the Python package
"""

##########################################################################
## Imports
##########################################################################

import os
import codecs

from setuptools import setup
from setuptools import find_packages


##########################################################################
## Configuration
##########################################################################

NAME         = "jupyteranalytics"
DESCRIPTION  = "Adds Google Analytics to Jupyter notebooks"
AUTHOR       = "PingThings, Inc."
EMAIL        = "support@pingthings.io"
LICENSE      = "BSD-3-Clause"
REPOSITORY   = "https://github.com/PingThingsIO/jupyter-analytics"
PACKAGE      = "jupyteranalytics"
KEYWORDS     = ("Google Analytics", "notebooks", "Jupyter", "JupyterHub")

## Define the classifiers
## See https://pypi.python.org/pypi?%3Aaction=list_classifiers
CLASSIFIERS  = (
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',
    'Framework :: Jupyter',
    'Intended Audience :: Developers',
    'Intended Audience :: Science/Research',
    'Intended Audience :: System Administrators',
    'License :: OSI Approved :: BSD License',
    'Natural Language :: English',
    'Operating System :: OS Independent',
    'Programming Language :: Python'
)

PROJECT      = os.path.abspath(os.path.dirname(__file__))
REQUIRE_PATH = "requirements.txt"
VERSION_PATH = os.path.join(PACKAGE, "version.py")
PKG_DESCRIBE = "README.md"
EXCLUDES     = ("tests", "docs", "fixtures")

##########################################################################
## Helper Functions
##########################################################################

def read(*parts):
    """
    Assume UTF-8 encoding and return the contents of the file located at the
    absolute path from the REPOSITORY joined with *parts.
    """
    with codecs.open(os.path.join(PROJECT, *parts), 'rb', 'utf-8') as f:
        return f.read()


def get_version(path=VERSION_PATH):
    """
    Reads the python file defined in the VERSION_PATH to find the get_version
    function, and executes it to ensure that it is loaded correctly. Separating
    the version in this way ensures no additional code is executed.
    """
    namespace = {}
    exec(read(path), namespace)
    return namespace['get_version'](short=True)


def get_requires(path=REQUIRE_PATH):
    """
    Yields a generator of requirements as defined by the REQUIRE_PATH which
    should point to a requirements.txt output by `pip freeze`.
    """
    for line in read(path).splitlines():
        line = line.strip()
        if line and not line.startswith('#'):
            yield line


def get_description_type(path=PKG_DESCRIBE):
    """
    Returns the long_description_content_type based on the extension of the
    package describe path (e.g. .txt, .rst, or .md).
    """
    _, ext = os.path.splitext(path)
    return {
        ".rst": "text/x-rst",
        ".txt": "text/plain",
        ".md": "text/markdown",
    }[ext]


##########################################################################
## Define the configuration
##########################################################################

config = {
    "name": NAME,
    "version": get_version(),
    "description": DESCRIPTION,
    "long_description": read(PKG_DESCRIBE),
    "long_description_content_type": get_description_type(PKG_DESCRIBE),
    "classifiers": list(CLASSIFIERS),
    "keywords": list(KEYWORDS),
    "license": LICENSE,
    "author": AUTHOR,
    "author_email": EMAIL,
    "maintainer": AUTHOR,
    "maintainer_email": EMAIL,
    "project_urls": {
        "Download": "{}/tarball/v{}".format(REPOSITORY, get_version()),
        "Source": REPOSITORY,
        "Tracker": "{}/issues".format(REPOSITORY),
    },
    "download_url": "{}/tarball/v{}".format(REPOSITORY, get_version()),
    "packages": find_packages(where=PROJECT, exclude=EXCLUDES),
    "include_package_data": True,
    "package_data": {
        "jupyteranalytics": ["static/*"],
    },
    "data_files": [
        # Automatic `jupyter nbextension install --sys-prefix`
        ("share/jupyter/nbextensions/jupyteranalytics", [
            "jupyteranalytics/static/main.js",
        ]),
        # Automatic `jupyter nbextension enable --sys-prefix`
        ("etc/jupyter/nbconfig/", [
            "jupyter-config/nbconfig/common.json"
        ]),
        #  Automatic `jupyter serverextension enable --sys-prefix`
        ("etc/jupyter/jupyter_notebook_config.d", [
            "jupyter-config/jupyter_notebook_config.d/jupyteranalytics.json"
        ])
    ],
    "zip_safe": False,
    "install_requires": list(get_requires()),
    "python_requires": ">=3.4, <4",
}


##########################################################################
## Run setup script
##########################################################################

if __name__ == '__main__':
    setup(**config)

